
;===================================================================================================
; Copyright (C) KolibriOS team 2004-2011. All rights reserved.
; Distributed under terms of the GNU General Public License
;===================================================================================================
; THIS SOFTWARE IS PROVIDED BY <copyright holders> ''AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
; INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
; PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL <copyright holders> BE LIABLE FOR ANY DIRECT, INDIRECT,
; INCIDENTAL, SPECIAL,EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
; SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
; AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
; SUCH DAMAGE.


$Revision: 2288 $

struc VBE_VGAInfo {
  .VESASignature	  dd ?	  ; char
  .VESAVersion		  dw ?	  ; short
  .OemStringPtr 	  dd ?	  ; char *
  .Capabilities 	  dd ?	  ; ulong
  .VideoModePtr 	  dd ?	  ; ulong
  .TotalMemory		  dw ?	  ; short
  ; VBE 2.0+
  .OemSoftwareRev	  db ?	  ; short
  .OemVendorNamePtr	  dw ?	  ; char *
  .OemProductNamePtr	  dw ?	  ; char *
  .OemProductRevPtr	  dw ?	  ; char *
  .reserved		  rb 222  ; char
  .OemData		  rb 4096;256  ; char
}

struc VBE_ModeInfo {
  .ModeAttributes	  dw ?	  ; short
  .WinAAttributes	  db ?	  ; char
  .WinBAttributes	  db ?	  ; char
  .WinGranularity	  dw ?	  ; short
  .WinSize		  dw ?	  ; short
  .WinASegment		  dw ?	  ; ushort
  .WinBSegment		  dw ?	  ; ushort
  .WinFuncPtr		  dd ?	  ; void *
  .BytesPerScanLine	  dw ?	  ; short
  .XRes 		  dw ?	  ; short
  .YRes 		  dw ?	  ; short
  .XCharSize		  db ?	  ; char
  .YCharSize		  db ?	  ; char
  .NumberOfPlanes	  db ?	  ; char
  .BitsPerPixel 	  db ?	  ; char
  .NumberOfBanks	  db ?	  ; char
  .MemoryModel		  db ?	  ; char
  .BankSize		  db ?	  ; char
  .NumberOfImagePages	  db ?	  ; char
  .res1 		  db ?	  ; char

  ; next 9 fields valid for DirectColor model only:
  .RedMaskSize		  db ?	  ; char
  .RedFieldPosition	  db ?	  ; char
  .GreenMaskSize	  db ?	  ; char
  .GreenFieldPosition	  db ?	  ; char
  .BlueMaskSize 	  db ?	  ; char
  .BlueFieldPosition	  db ?	  ; char
  .RsvedMaskSize	  db ?	  ; char
  .RsvedFieldPosition	  db ?	  ; char
  .DirectColorModeInfo	  db ?	  ; char ; MISSED IN THIS TUTORIAL!! SEE ABOVE

  ; VBE 2.0+
  .PhysBasePtr		  dd ?	  ; ulong
  .OffScreenMemOffset	  dd ?	  ; ulong
  .OffScreenMemSize	  dw ?	  ; short
  ; VBE 3.0+
  .LinbytesPerScanLine	  dw ?	  ; short
  .BankNumberOfImagePages db ?	  ; char
  .LinNumberOfImagePages  db ?	  ; char
  .LinRedMaskSize	  db ?	  ; char
  .LinRedFieldPosition	  db ?	  ; char
  .LingreenMaskSize	  db ?	  ; char
  .LinGreenFieldPosition  db ?	  ; char
  .LinBlueMaskSize	  db ?	  ; char
  .LinBlueFieldPosition   db ?	  ; char
  .LinRsvdMaskSize	  db ?	  ; char
  .LinRsvdFieldPosition   db ?	  ; char
  .MaxPixelClock	  dd ?	  ; ulong
  .res2 		  rb 4096
}

virtual at $3000
    vi			VBE_VGAInfo
    mi			VBE_ModeInfo
end virtual

    edid_buff		= 0x3000

    modes_table 	= 0xA000

    monitorAR		dw 0
    edid_1stEntry	dw 0		; ptr where 1st edid entry starts, entry size = 6bytes
    edid_version	dw 0
    edid_sizes		dw 0		; width (lowest byte) and height  //  OR aspect ratio
    edid_timings	dw 0		; standart & established timings
    vbe_version 	dw 0
    vbe_count		db 0		; number of vbe modes found
    edid_count		db 0		; number of edid entries describing monitor resolutions
    vbe_selectedMode	dw 0		; byte offset of an entry (relative to 'modes_tables' array)
    vbe_mem		dw 0

    cursor_pos		dw 0		;   .
    home_cursor 	dw 0		; current shows rows a table
    end_cursor		dw 0		; end of position current shows rows a table
    scroll_start	dw 0		; start position of scroll bar
    scroll_end		dw 0		; end position of scroll bar
    size_of_step	equ 16
    long_v_table	equ 9		; long of visible video table
    scroll_area_size	equ (long_v_table-2)


;===================================================================================================
;    vbe_parseVidModes    ||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
;===================================================================================================

; return: array at 'modes_table' filled. Entry size = 'size_of_step' constant

vbe_parseVidModes:
	push	ds es fs
	xor	eax, eax
	mov	es, ax
	mov	ds, ax
	mov	[0x9014], eax			; vbe 1.2 bank switch

	; see if vbe2+ available
	mov	[cs:vbe_version], 0
	mov	[vi.VESASignature], 'VBE2'
	mov	[vi.VESAVersion], 0
	mov	ax, 0x4F00
	mov	di, vi
	int	0x10

	lfs	si, [vi.VideoModePtr]
	mov	bx, modes_table

	; VGA entry:
	mov	word [bx], 640
	mov	word [bx+2], 480
	mov	word [bx+6], 0x12
	mov	dword [bx+8], 0
	mov	dword [bx+12], 0
	;mov     byte [bx+8], 0
	;mov     dword [bx+10], -1

	add	bx, size_of_step
	add	[cs:vbe_count], 1

	; check if ax=4f00 failed
	test	ah, ah
	jnz	.done
	cmp	word [es:vi.VESAVersion], 0x200
	jb	.done

	mov	ax, [vi.VESAVersion]
	mov	[cs:vbe_version], ax

	add	ax, '00'
	mov	[s_vesa.ver], ah
	mov	[s_vesa.ver+2], al

.next_mode:
	mov	cx, [fs:si]
	cmp	cx, -1
	jz	.done

	or	cx, 1 shl 14
	mov	bp, cx				; bp = copy of videomode #
	mov	ax, 0x4F01
	mov	di, mi
	int	0x10
	test	ah, ah
	jnz	.done

	mov	ax, [mi.ModeAttributes]
	not	ax
	test	al, 1001'1001b			; supported, color, graphics & lfb flags
	jnz	.switch

	mov	cx, [mi.XRes]
	mov	dx, [mi.YRes]
	cmp	[mi.BitsPerPixel], 24		; It show only videomodes to have support 24 and 32 bpp
	jb	.switch
	cmp	cx, 640
	jb	.switch
	cmp	dx, 480
	jb	.switch

	mov	ax, [es:mi.BytesPerScanLine]
	mov	[bx+0], cx			; +0[2] : width
	mov	[bx+2], dx			; +2[2] : height
	mov	[bx+4], ax			; +4[2] : bytes per scanline
	mov	[bx+6], bp			; +6[2] : mode number with or without LFB bit (all set)

	mov	al, [mi.BitsPerPixel]
	mov	ah, [mi.MemoryModel]
	mov	edx, [mi.PhysBasePtr]
	mov	[bx+8], al			; +8[1] : bits per pixel
	mov	[bx+9], ah			; +9[1] : color/memory model
	mov	[bx+10], edx			; +10[4]  lfb address
	mov	word [bx+14], 0 		; +14[1]  non zero value if mode supported by monitor
						; +15[1]  ? (could be refresh rate)

	cmp	[mi.MemoryModel], 4		; any PackedPixel is OK if we only support 24 & 32 bps
	jz	@f
	cmp	[mi.MemoryModel], 6		; only PacketPixel & DirectColor supported
	jnz	.switch


	; TODO:   right here add mask & position check for direct color

@@:
	test	edx, edx			; lfb sanity verification
	jz	.switch

	add	bx, size_of_step		; += size of one entry
	inc	[cs:vbe_count]
	jz	.cnt_overflow
.switch:
	add	si, 2
	ja	.next_mode
	jmp	.done

.cnt_overflow:
	dec	[cs:vbe_count]
.done:

	mov	word [bx], -1			; end video table
	mov	word [end_cursor], bx		; save end cursor position

	pop	fs es ds
	ret

;===================================================================================================
;|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
;===================================================================================================

int2str:
	dec	bl
	jz	@f
	xor	edx, edx
	div	ecx
	push	edx
	call	int2str
	pop	eax
    @@:
	or	al, 0x30
	mov	[ds:di], al
	inc	di
	ret

int2strnz:
	cmp	eax, ecx
	jb	@f
	xor	edx, edx
	div	ecx
	push	edx
	call	int2strnz
	pop	eax
    @@:
	or	al, 0x30
	mov	[es:di], al
	inc	di
	ret

;-------------------------------------------------------
;Write message about incorrect v_mode and write message about jmp on swith v_mode
v_mode_error:
	_setcursor 19,2
	mov	si, fatalsel
	call	printplain
	_setcursor 20,2
	mov	si, pres_key
	call	printplain
	xor	eax, eax
	int	16h
	jmp	cfgmanager.d
;-------------------------------------------------------
;




;===================================================================================================
; Dropdown list with videomodes
;===================================================================================================

draw_current_vmode:

	push	0
	pop	es
	mov	si, word [cursor_pos]
	push	gs 0
	pop	gs
	mov	bx, [cs:vbe_selectedMode]
	add	bx, modes_table



if defined extended_primary_loader
	mov	di, config_file_variables
else
	mov	di, loader_block_error
end if


	;movzx   eax, word[es:si+0]
	movzx	eax, word [gs:bx]
	mov	ecx, 10
	push	bx
	call	int2strnz
	pop	bx

	mov	byte[es:di], 'x'
	inc	di
	;movzx   eax, word[es:si+2]
	movzx	eax, word [gs:bx+2]
	push	bx
	call	int2strnz
	pop	bx

	mov	byte[es:di], 'x'
	inc	di
	;movzx   eax, word[es:si+8]
	movzx	eax, byte [gs:bx+8]
	push	bx
	call	int2strnz
	mov	dword[es:di], 0x00000d0a
	pop	bx


	sub	bx, modes_table
	mov	[cs:vbe_selectedMode], bx


if defined extended_primary_loader
	mov	si, config_file_variables
else
	mov	si, loader_block_error
end if

	push	ds
	push	es
	pop	ds
	call	printplain
	pop	ds
	jmp	.exit


;.no_vesa_0x13:
;        mov     si, mode0  ; ega/cga
;        jmp     .print
.no_vesa_0x12:
	mov	si, mode9  ;vga
.print:
	call	printplain
.exit:
	pop	gs
	ret

;===================================================================================================
;
;===================================================================================================

check_first_parm:
if defined extended_primary_loader
	mov	cx, [number_vm]
	jcxz	.novbemode
	mov	si, modes_table
.findvbemode:
	cmp	[es:si+6], cx
	jnz	@f
	cmp	word [es:si+8], 32
	je	.ok_found_mode
	cmp	word [es:si+8], 24
	je	.ok_found_mode
@@:
	add	si, size_of_step
	cmp	word [es:si], -1
	jnz	.findvbemode
.novbemode:
	mov	ax, [x_save]
	test	ax, ax
	jz	.zerro
	mov	bx, [y_save]
	mov	si, modes_table
	call	.loops
	test	ax, ax
	jz	.ok_found_mode
else
	mov	si, word [preboot_graph]
	test	si, si
	jnz	.no_zero	 ;if no zero
end if
.zerro:
;        mov     ax,modes_table
;        mov     word [cursor_pos],ax
;        mov     word [home_cursor],ax
;        mov     word [preboot_graph],ax
;SET default video of mode first probe will fined a move of work 1024x768@32

	mov	ax, 1024
	mov	bx, 768
	mov	si, modes_table
	call	.loops
	test	ax, ax
	jz	.ok_found_mode
	mov	ax, 800
	mov	bx, 600
	mov	si, modes_table
	call	.loops
	test	ax, ax
	jz	.ok_found_mode
	mov	ax, 640
	mov	bx, 480
	mov	si, modes_table
	call	.loops
	test	ax, ax
	jz	.ok_found_mode

	mov	si, modes_table
if ~ defined extended_primary_loader
	jmp	.ok_found_mode



.no_zero:
	mov	bp, word [number_vm]
	cmp	bp, word [es:si+6]
	jz	.ok_found_mode


	mov	ax, word [x_save]
	mov	bx, word [y_save]
	mov	si, modes_table
	call	.loops
	test	ax, ax
	jz	.ok_found_mode

	mov	si, modes_table
;        cmp     ax,modes_table
;        jb      .zerro           ;check on correct if bellow
;        cmp     ax,word [end_cursor]
;        ja      .zerro           ;check on correct if anymore
end if

.ok_found_mode:
	mov	word [home_cursor], si
;        mov     word [cursor_pos],si
	mov	word [preboot_graph], si
	mov	ax, si

	mov	ecx, long_v_table

.loop:
	add	ax, size_of_step		       ; <--
	cmp	ax, word [end_cursor]
	jae	.next_step
	loop	.loop
.next_step:
	sub	ax, size_of_step;*long_v_table          ; <-- overflow bug
	cmp	ax, modes_table
	jae	@f
	mov	ax, modes_table
@@:

	mov	word [home_cursor], ax
	mov	si, [preboot_graph]
	mov	word [cursor_pos], si

	push	word [es:si]
	pop	word [x_save]
	push	word [es:si+2]
	pop	word [y_save]
	push	word [es:si+6]
	pop	word [number_vm]

	ret
;;;;;;;;;;;;;;;;;;;;;;;;;;;
.loops:
	cmp	ax, word [es:si]
	jne	.next
	cmp	bx, word [es:si+2]
	jne	.next
	cmp	word [es:si+8], 32
	je	.ok
	cmp	word [es:si+8], 24
	je	.ok
.next:
	add	si, size_of_step
	cmp	word [es:si], -1
	je	.exit
	jmp	.loops
.ok:
	xor	ax, ax
	ret
.exit:
	or	ax, -1
	ret


;-----------------------------------------------------------------------------

;default_vmode:
	
;===================================================================================================
;
;===================================================================================================

draw_vmodes_table:
	_setcursor 9, 2
	mov	si, gr_mode
	call	printplain

	mov	si, _st
	call	printplain

	push	word [cursor_pos]
	pop	ax
	push	word [home_cursor]
	pop	si
	mov	cx, si

	cmp	ax, si
	je	.ok
	jb	.low


	add	cx, size_of_step*long_v_table
	
	cmp	ax, cx
	jb	.ok
	
	sub	cx, size_of_step*long_v_table
	add	cx, size_of_step
	cmp	cx, word[end_cursor]
	jae	.ok
	add	si, size_of_step
	push	si
	pop	word [home_cursor]
	jmp	.ok


.low:
	sub	cx, size_of_step
	cmp	cx, modes_table
	jb	.ok
	push	cx
	push	cx
	pop	word [home_cursor]
	pop	si


.ok:
; calculate scroll position
	push	si
	mov	ax, [end_cursor]
	sub	ax, modes_table
	mov	bx, size_of_step
	cwd
	div	bx
	mov	si, ax		; si = size of list
	mov	ax, [home_cursor]
	sub	ax, modes_table
	cwd
	div	bx
	mov	di, ax
	mov	ax, scroll_area_size*long_v_table
	cwd
	div	si
	test	ax, ax
	jnz	@f
	inc	ax
@@:
	cmp	al, scroll_area_size
	jb	@f
	mov	al, scroll_area_size
@@:
	mov	cx, ax
; cx = scroll height
; calculate scroll pos
	xor	bx, bx		; initialize scroll pos
	sub	al, scroll_area_size+1
	neg	al
	sub	si, long_v_table-1
	jbe	@f
	mul	di
	div	si
	mov	bx, ax
@@:
	inc	bx
	imul	ax, bx, size_of_step
	add	ax, [home_cursor]
	mov	[scroll_start], ax
	imul	cx, size_of_step
	add	ax, cx
	mov	[scroll_end], ax
	pop	si
	mov	bp, long_v_table	      ;show rows
.@@_next_bit:
;clear cursor
	mov	ax, '  '
	mov	word[ds:_r1+21], ax
	mov	word[ds:_r1+50], ax

	mov	word[ds:_r2+21], ax
	mov	word[ds:_r2+45], ax

	mov	word[ds:_rs+21], ax
	mov	word[ds:_rs+46], ax
; draw string
	cmp	word [es:si+6], 0x12
	je	.show_0x12
	cmp	word [es:si+6], 0x13
	je	.show_0x13

	movzx	eax, word[es:si]
	cmp	ax, -1
	je	.@@_end
	mov	di, _rs+23
	mov	ecx, 10
	mov	bl, 4
	call	int2str
	movzx	eax, word[es:si+2]
	inc	di
	mov	bl, 4
	call	int2str

	movzx	eax, byte [es:si+8]
	inc	di
	mov	bl, 2
	call	int2str

	cmp	si, word [cursor_pos]
	jne	.next
;draw   cursor
	mov	word[ds:_rs+21], '>>'
	mov	word[ds:_rs+46], '<<'




.next:
	push	si
	mov	si, _rs
.@@_sh:
; add to the string pseudographics for scrollbar
	pop	bx
	push	bx
	mov	byte [si+53], ' '
	cmp	bx, [scroll_start]
	jb	@f
	cmp	bx, [scroll_end]
	jae	@f
	mov	byte [si+53], 0xDB ; filled bar
@@:
	push	bx
	add	bx, size_of_step
	cmp	bx, [end_cursor]
	jnz	@f
	mov	byte [si+53], 31 ; 'down arrow' symbol
@@:
	sub	bx, [home_cursor]
	cmp	bx, size_of_step*long_v_table
	jnz	@f
	mov	byte [si+53], 31 ; 'down arrow' symbol
@@:
	pop	bx
	cmp	bx, [home_cursor]
	jnz	@f
	mov	byte [si+53], 30 ; 'up arrow' symbol
@@:
	call	printplain
	pop	si
	add	si, size_of_step

	dec	bp
	jnz	.@@_next_bit

.@@_end:
	mov	si, _bt
	call	printplain
	ret
.show_0x13:
	push	si

	cmp	si, word [cursor_pos]
	jne	@f
	mov	word[ds:_r1+21], '>>'
	mov	word[ds:_r1+50], '<<'
@@:
	mov	si, _r1
	jmp	.@@_sh
.show_0x12:
	push	si
	cmp	si, word [cursor_pos]
	jne	@f

	mov	word[ds:_r2+21], '>>'
	mov	word[ds:_r2+45], '<<'
@@:
	mov	si, _r2
	jmp	.@@_sh

;-----------------------------------------------------------------------------
;Clear arrea of current video page (0xb800)
clear_vmodes_table:
	pusha
       ; draw frames
	push	es
	push	0xb800
	pop	es
	mov	di, 1444
	xor	ax, ax
	mov	ah, 1*16+15
	mov	cx, 70
	mov	bp, 12
.loop_start:
	rep stosw
	mov	cx, 70
	add	di, 20
	dec	bp
	jns	.loop_start
	pop	es
	popa
	ret

;-----------------------------------------------------------------------------



;        mov     dword[es:0x9018],0x000A0000
;        ret



